C---------------------------------------------------------------------
      SUBROUTINE CXFH(DRX,NRAMAX,IPUT,ENGY,VDEPTH,RPUT,FFGN1,FFGN2)
C---------------------------------------------------------------------
C   Calculate regular and singular wave functions 
C   of Schroedinger equations with complex potential for complex energy
C   Modified by M. ICHIMURA 2016.6
C----------------------------------------------------------------------
C  INPUTS
C    ENGY  : Complex energy
C    VDEPTH: Complex potential depth
C    NRAMAX: Number of mesh points for radius
c    DRX   : Mesh size for radius
C  OUTPUTS
C    FFGN1(NR): Regular solution
C    FFGN2(NR): Singular solution
C----------------------------------------------------------------------
      IMPLICIT NONE
C INPUTS: Through arguments
      INTEGER IPUT(1:3)
      REAL    RPUT(1:21)
      INTEGER NRAMAX
      REAL    DRX
      COMPLEX ENGY,VDEPTH
C INPUTS: Through IPUT, RPUT, ENGY and VDEPTH
      INTEGER LMOM,IS2,J2
      REAL*8  WSMAS,WMASS
      REAL*8  WR0,WA0,WV0,WS0,WRV,WAV,WRI,WAI
      REAL*8  WLS,WRSO,WASO,WRC,WZZ,WREFF,WAEFF,BDIF
      REAL*8  BEFM,SPWD
      REAL    ENGYR,ENGYI,ENGYABS
      REAL*8  ENGYDR,ENGYDI,ENGYDABS
      REAL    VDEP,WDEP
      REAL*8  VDEPD,WDEPD
C OUTPUTS:
      COMPLEX*16 FFGN1(402)
      COMPLEX*16 FFGN2(402)
C VARIABLES:
      INTEGER I,NR2,IP,IP2,JRO
      REAL*8  FMU,DRD,FLMOM,FLMOM1
      REAL*8  WKA,WKPL,WKMI
      REAL*8  WDRO,WR
      REAL*8  RADI,RWV,RWI,RSO,RCO,REFF
      REAL*8  EX,EXV,EXI,EXSO,EXEFF
      REAL*8  WFCR(402),WFIVS(402)
      REAL*8  WFSR(402),WFC(402)
      REAL*8  MURATIO(402)
      REAL*8  FJS,FIS,FLNS
      REAL*8  WRO
      REAL*8  DRO12,DRO56
      COMPLEX*16 WKC
      COMPLEX WETA
      COMPLEX*16 WLOCAL(402),EPOT(402)
      COMPLEX*16 FFR(402,2)
      COMPLEX*16 FAC1,FAC2,FAC3
      COMPLEX*16 DFF1RT,DFF2RT
      COMPLEX*16 ROEND,ROEND1,EXEND,EXEND1
      COMPLEX*16 WRONSK
C CONSTANTS:
      real*8 hc/197.326968d0/
      real*8 cpi/3.141592654d0/
      real*8 AMU/931.494043d0/
      REAL*8 ALPHA
      parameter(alpha=1.0d0/137.036d0)
      REAL*8 TEST
      PARAMETER(TEST=21.0d0)
      COMPLEX*16 WI0
      PARAMETER(WI0=(0.0d0,1.0d0))
C EXTERNAL:
      REAL   SIGMA1
      REAL   SIGL
      REAL*8 SIGLD
C
      NR2=NRAMAX+2
      LMOM=IPUT(1)
      FLMOM=REAL(LMOM)
      FLMOM1=REAL(LMOM*(LMOM+1))
      IS2=IPUT(2)
      J2=IPUT(3)
      DRD=DRX
      WSMAS=RPUT(1)
      WMASS=RPUT(2)
      WR0=RPUT(3)
      WA0=RPUT(4)
      WV0=RPUT(5)
      WS0=RPUT(6)
      WRV=RPUT(7)
      WAV=RPUT(8)
      WRI=RPUT(9)
      WAI=RPUT(10)
      WLS=RPUT(11)
      WRSO=RPUT(12)
      WASO=RPUT(13)
      WZZ=RPUT(14)
      WREFF=RPUT(15)
      WAEFF=RPUT(16)
      BDIF=RPUT(17)
      WRC=RPUT(21)
C  ENGY
      ENGYR   = REAL(ENGY)
      ENGYI   = AIMAG(ENGY)
      ENGYABS = ABS(ENGY)
      ENGYDR  = DBLE(ENGYR)
      ENGYDI  = DBLE(ENGYI)
      ENGYDABS= DBLE(ENGYABS)
C  VDEPTH
      VDEP  = REAL(VDEPTH)
      WDEP  = AIMAG(VDEPTH)
      VDEPD = DBLE(VDEP)
      WDEPD = DBLE(WDEP)
C
      DO 5 I=1,NR2,1
      FFGN1(I)=(0.0,0.0)
 5    CONTINUE
C
      DO 7 I=1,NR2,1
      FFGN2(I)=(0.0,0.0)
 7    CONTINUE
C
      FMU=WSMAS*WMASS/(WSMAS+WMASS)*AMU
      WKA=SQRT(2.0*FMU*ENGYDABS/HC**2)
      WKPL=SQRT((1.0d0+ENGYDR/ENGYDABS)/2.0d0)
      WKMI=SQRT((1.0d0-ENGYDR/ENGYDABS)/2.0d0)
      IF(ENGYDR.GE.0.0d0) THEN
      IF(ENGYDI.GE.0.0d0) THEN
      WKC=WKA*CMPLX(WKPL,WKMI)
      ELSE IF(ENGYDI.LT.0.0d0) THEN
      WKC=WKA*CMPLX(WKPL,-WKMI)
      END IF
      ELSE IF(ENGYDR.LT.0.0D0) THEN
      IF(ENGYDI.GE.0.0d0) THEN
      WKC=WKA*CMPLX(WKPL,WKMI)
      ELSE IF(ENGYDI.LT.0.0D0) THEN
      WKC=WKA*CMPLX(-WKPL,WKMI)
      END IF
      END IF
C
      WDRO=WKA*DRD
      WETA=WZZ*ALPHA*FMU/HC/WKC
      RADI=WR0*WMASS**0.3333333
      RWV=WRV*WMASS**0.3333333
      RWI=WRI*WMASS**0.3333333
      RSO=WRSO*WMASS**0.3333333
      RCO=WRC*WMASS**0.3333333
      REFF=WREFF*WMASS**0.3333333
C
      WR=DRD
      DO 10 I=1,NR2
      EX=EXP((WR-RADI)/WA0)
      EXV=EXP((WR-RWV)/WAV)
      EXI=EXP((WR-RWI)/WAI)
      EXSO=EXP((WR-RSO)/WASO)
      EXEFF=EXP((WR-REFF)/WAEFF)
      WFCR(I)=1.0/(1.0+EX)
      WFIVS(I)=WV0/(WV0+WS0)*1.0/(1.0+EXI)
     1          +WS0/(WV0+WS0)*4.0*EXV/((1.0+EXV)*(1.0+EXV))
      WFSR(I)=EXSO/((1.0+EXSO)*(1.0+EXSO))/WR
C
C Coulomb potential
      IF(WR.LE.RCO) THEN
      WFC(I)=0.7199262*WZZ*(3.0-WR*WR/(RCO*RCO))/RCO
      ELSE IF(WR.GT.RCO) THEN
      WFC(I)=1.439852*WZZ/WR
      END IF
      MURATIO(I)=1.0-BDIF/(1.0+EXEFF)
      WR=WR+DRD
 10   CONTINUE
C
      FJS=0.5*REAL(J2)
      FIS=0.5*REAL(IS2)
      FLNS=FJS*(FJS+1.0)-FLMOM*(FLMOM+1.0)-FIS*(FIS+1.0)
C
      DO 20 I=1,NR2
      WLOCAL(I)=-VDEPD*WFCR(I)-WI0*WDEPD*WFIVS(I)
     &         -2.0*WLS/WASO*FLNS*WFSR(I)+WFC(I)
 20   CONTINUE
C
      WRO=WDRO
      DO 30 I=1,NR2
      EPOT(I)=MURATIO(I)*(ENGY/ABS(ENGY)-WLOCAL(I)/ABS(ENGY))
     &      -FLMOM1/WRO**2
      WRO=WRO+WDRO
  30  CONTINUE
C
      DRO12=1.0/12.0*WDRO**2
      DRO56=5.0/6.0*WDRO**2
C
      DO 40 I=1,NR2,1
      FFR(I,1)=(0.0,0.0)
 40   CONTINUE
C
      FFR(1,1)=WDRO**(LMOM+1)
      FAC2=2.0-DRO56*EPOT(1)
      FAC3=1.0+DRO12*EPOT(2)
      FFR(2,1)=FFR(1,1)*FAC2/FAC3
C
      DO 200 I=1,NRAMAX,1
      FAC1=1.0+DRO12*EPOT(I)
      FAC2=2.0-DRO56*EPOT(I+1)
      FAC3=1.0+DRO12*EPOT(I+2)
      FFR(I+2,1)=(FFR(I+1,1)*FAC2-FFR(I,1)*FAC1)/FAC3
      IF(ABS(FFR(I+2,1)).LT.TEST) THEN
      GOTO 200
      ELSE IF(ABS(FFR(I+2,1)).GE.TEST) THEN
      GOTO 185
      END IF
  185 IP2=I+2
      DO 190 IP=1,IP2
      FFR(IP,1)=FFR(IP,1)/TEST
  190 CONTINUE
  200 CONTINUE
C
      DO 300 I=1,NR2,1
      FFR(I,2)=(0.0,0.0)
 300  CONTINUE
C
      SIGL=SIGMA1(LMOM,WETA)
      SIGLD=DBLE(SIGL)
      JRO=NR2
 310  ROEND=WKC*DRD*REAL(JRO)
      ROEND1=WKC*DRD*REAL(JRO-1)
      EXEND =ROEND-WETA*LOG(2.0*ROEND)-0.5*FLMOM*CPI+SIGLD
      EXEND1=ROEND1-WETA*LOG(2.0*ROEND1)-0.5*FLMOM*CPI+SIGLD
      IF((DIMAG(EXEND).GE.87.0).OR.
     &   (DIMAG(EXEND1).GE.87.0)) THEN
      JRO=JRO-1
      IF(JRO.LE.5) THEN
      WRITE(6,*) ' ERROR IN CXGN '
      GOTO 999
      END IF
      GOTO 310
      END IF
      FFR(JRO,2)=EXP(WI0*EXEND)
      FFR(JRO-1,2)=EXP(WI0*EXEND1)
C
      DO 320 I=JRO,3,-1
      FAC1=1.D0+DRO12*EPOT(I)
      FAC2=2.D0-DRO56*EPOT(I-1)
      FAC3=1.D0+DRO12*EPOT(I-2)
      FFR(I-2,2)=(FFR(I-1,2)*FAC2-FFR(I,2)*FAC1)/FAC3
 320  CONTINUE
C
      DFF1RT=2.0*FFR(JRO-4,1)-16.0*FFR(JRO-3,1)
     &      +16.0*FFR(JRO-1,1)-2.0*FFR(JRO,1)
      DFF1RT=DFF1RT/24.0/DRD
      DFF2RT=2.0*FFR(JRO-4,2)-16.0*FFR(JRO-3,2)
     &      +16.0*FFR(JRO-1,2)-2.0*FFR(JRO,2)
      DFF2RT=DFF2RT/24.0/DRD
      WRONSK=FFR(JRO-2,1)*DFF2RT-DFF1RT*FFR(JRO-2,2)
C
      DO 250 I=1,NR2,1
      FFGN1(I)=FFR(I,1)*SQRT(MURATIO(I))
 250  CONTINUE
C
      DO 330 I=1,NR2,1
      FFGN2(I)=FFR(I,2)/WRONSK*SQRT(MURATIO(I))
 330  CONTINUE
C
      RETURN
C
 999  STOP
      END
C-------------------------------------------------------------------
      FUNCTION SIGMA1(L,ETA)
C-----------------------------------------------------------------
      IMPLICIT NONE
C INPUT:
      INTEGER L
      COMPLEX ETA
C OUTPUT:
      REAL SIGMA1
C VARIABLES:
      INTEGER K,N
      COMPLEX Z
      COMPLEX LNGAM
C CONSTANT:
      real*8 cpi/3.141592654d0/
      COMPLEX WI0
      PARAMETER(WI0=(0.0,1.0))
C EXTERNAL:
      COMPLEX ARGL2
C
      Z=REAL(L+1)+WI0*ETA
      N=2*(INT(ABS(Z))+1)
      LNGAM=0.5*(1.0-REAL(N))*LOG(2.0*CPI)
     &      +(Z-0.5)*LOG(REAL(N))
      DO 30 K=0,N-1,1
      LNGAM=LNGAM+ARGL2((Z+REAL(K))/REAL(N))
 30   CONTINUE
      SIGMA1=AIMAG(LNGAM)
      RETURN
      END
C---------------------------------------------------------------
      FUNCTION ARGL2(Z1)
C---------------------------------------------------------------
      IMPLICIT NONE
C INPUT:
      COMPLEX Z1
C OUTPUT:
      COMPLEX ARGL2
C VARIABLES:
      INTEGER N
      REAL TSETA1(100)
      COMPLEX VAR
C
      TSETA1(2)=0.6449341
      TSETA1(3)=0.2020569
      TSETA1(4)=0.08232323
      TSETA1(5)=0.03692776
      TSETA1(6)=0.01734306
      TSETA1(7)=0.008349277
      TSETA1(8)=0.004077356
      TSETA1(9)=0.002008393
      TSETA1(10)=0.0009945751
      TSETA1(11)=0.0004941886
      TSETA1(12)=0.0002460866
      TSETA1(13)=0.0001227133
      TSETA1(14)=0.00006124814
      TSETA1(15)=0.00003058824
      TSETA1(16)=0.00001528226
      TSETA1(17)=0.000007637198
      TSETA1(18)=0.000003817293
      TSETA1(19)=0.000001908213
      TSETA1(20)=0.0000009539620
C
      VAR=-LOG(Z1)-LOG(1.0+Z1)+Z1*(1.0-0.5772157)
      DO 20 N=2,20,1
      VAR=VAR+(-1)**N*TSETA1(N)*(Z1)**N/REAL(N)
 20   CONTINUE
      ARGL2=VAR
c
      RETURN
      END
